@@ -1,9 +1,3 @@
-"""
-.. testsetup::
-
-    from compas.geometry import Vector
-
-"""
 from __future__ import print_function
 from __future__ import absolute_import
 from __future__ import division
@@ -21,8 +15,6 @@ from compas.geometry import transform_vectors
 
 from compas.geometry.primitives import Primitive
 
-__all__ = ['Vector']
-
 
 class Vector(Primitive):
     """A vector is defined by XYZ components and a homogenisation factor.
@@ -38,20 +30,17 @@ class Vector(Primitive):
 
     Attributes
     ----------
-    data : dict
-        The data representation of the vector.
     x : float
-        The component along the X axis of the vector.
+        The X coordinate of the point.
     y : float
-        The component along the Y axis of the vector.
+        The Y coordinate of the point.
     z : float
-        The component along the Z axis of the vector.
+        The Z coordinate of the point.
     length : float, read-only
-        The length of the vector.
+        The length of this vector.
 
     Examples
     --------
-
     >>> u = Vector(1, 0, 0)
     >>> v = Vector(0, 1, 0)
     >>> u
@@ -79,8 +68,8 @@ class Vector(Primitive):
 
     __slots__ = ['_x', '_y', '_z']
 
-    def __init__(self, x, y, z=0):
-        super(Vector, self).__init__()
+    def __init__(self, x, y, z=0.0, **kwargs):
+        super(Vector, self).__init__(**kwargs)
         self._x = 0.0
         self._y = 0.0
         self._z = 0.0
@@ -88,6 +77,22 @@ class Vector(Primitive):
         self.y = y
         self.z = z
 
+    # ==========================================================================
+    # data
+    # ==========================================================================
+
+    @property
+    def DATASCHEMA(self):
+        """:class:`schema.Schema` : Schema of the data representation."""
+        from schema import Schema
+        from compas.data import is_float3
+        return Schema(is_float3)
+
+    @property
+    def JSONSCHEMANAME(self):
+        """str : Name of the schema of the data representation in JSON format."""
+        return 'vector'
+
     @property
     def data(self):
         """dict : The data dictionary that represents the vector."""
@@ -99,9 +104,33 @@ class Vector(Primitive):
         self.y = data[1]
         self.z = data[2]
 
+    @classmethod
+    def from_data(cls, data):
+        """Construct a vector from a data dict.
+
+        Parameters
+        ----------
+        data : dict
+            The data dictionary.
+
+        Returns
+        -------
+        :class:`compas.geometry.Vector`
+            The vector constructed from the provided data.
+
+        Examples
+        --------
+        >>> Vector.from_data([0.0, 0.0, 1.0])
+        Vector(0.000, 0.000, 1.000)
+        """
+        return cls(*data)
+
+    # ==========================================================================
+    # properties
+    # ==========================================================================
+
     @property
     def x(self):
-        """float : The X coordinate of the point."""
         return self._x
 
     @x.setter
@@ -110,7 +139,6 @@ class Vector(Primitive):
 
     @property
     def y(self):
-        """float : The Y coordinate of the point."""
         return self._y
 
     @y.setter
@@ -119,7 +147,6 @@ class Vector(Primitive):
 
     @property
     def z(self):
-        """float : The Z coordinate of the point."""
         return self._z
 
     @z.setter
@@ -128,7 +155,6 @@ class Vector(Primitive):
 
     @property
     def length(self):
-        """float: The length of this vector."""
         return length_vector(self)
 
     # ==========================================================================
@@ -176,7 +202,7 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        other : :class:`compas.geometry.Vector` or list
+        other : [float, float, float] | :class:`compas.geometry.Vector`
             The vector to compare.
 
         Returns
@@ -184,6 +210,7 @@ class Vector(Primitive):
         bool
             True if the vectors are equal.
             False otherwise.
+
         """
         return self.x == other[0] and self.y == other[1] and self.z == other[2]
 
@@ -192,13 +219,14 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        other : :class:`compas.geometry.Vector` or list
+        other : [float, float, float] | :class:`compas.geometry.Vector`
             The vector to add.
 
         Returns
         -------
         :class:`compas.geometry.Vector`
             The resulting vector.
+
         """
         return Vector(self.x + other[0], self.y + other[1], self.z + other[2])
 
@@ -207,13 +235,14 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        other : :class:`compas.geometry.Vector` or list
+        other : [float, float, float] | :class:`compas.geometry.Vector`
             The vector to subtract.
 
         Returns
         -------
         :class:`compas.geometry.Vector`
             The resulting new vector.
+
         """
         return Vector(self.x - other[0], self.y - other[1], self.z - other[2])
 
@@ -229,6 +258,7 @@ class Vector(Primitive):
         -------
         :class:`compas.geometry.Vector`
             The resulting new vector.
+
         """
         return Vector(self.x * n, self.y * n, self.z * n)
 
@@ -244,6 +274,7 @@ class Vector(Primitive):
         -------
         :class:`compas.geometry.Vector`
             The resulting new vector.
+
         """
         return Vector(self.x / n, self.y / n, self.z / n)
 
@@ -260,6 +291,7 @@ class Vector(Primitive):
         -------
         :class:`compas.geometry.Vector`
             A new point with raised coordinates.
+
         """
         return Vector(self.x ** n, self.y ** n, self.z ** n)
 
@@ -271,8 +303,13 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        other : :class:`compas.geometry.Vector` or list
+        other : [float, float, float] | :class:`compas.geometry.Vector`
             The vector to add.
+
+        Returns
+        -------
+        None
+
         """
         self.x += other[0]
         self.y += other[1]
@@ -284,8 +321,13 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        other : :class:`compas.geometry.Vector` or list
+        other : [float, float, float] | :class:`compas.geometry.Vector`
             The vector to subtract.
+
+        Returns
+        -------
+        None
+
         """
         self.x -= other[0]
         self.y -= other[1]
@@ -299,6 +341,11 @@ class Vector(Primitive):
         ----------
         n : float
             The multiplication factor.
+
+        Returns
+        -------
+        None
+
         """
         self.x *= n
         self.y *= n
@@ -312,6 +359,11 @@ class Vector(Primitive):
         ----------
         n : float
             The multiplication factor.
+
+        Returns
+        -------
+        None
+
         """
         self.x /= n
         self.y /= n
@@ -325,6 +377,11 @@ class Vector(Primitive):
         ----------
         n : float
             The power.
+
+        Returns
+        -------
+        None
+
         """
         self.x **= n
         self.y **= n
@@ -348,6 +405,7 @@ class Vector(Primitive):
         --------
         >>> Vector.Xaxis()
         Vector(1.000, 0.000, 0.000)
+
         """
         return cls(1.0, 0.0, 0.0)
 
@@ -364,6 +422,7 @@ class Vector(Primitive):
         --------
         >>> Vector.Yaxis()
         Vector(0.000, 1.000, 0.000)
+
         """
         return cls(0.0, 1.0, 0.0)
 
@@ -380,6 +439,7 @@ class Vector(Primitive):
         --------
         >>> Vector.Zaxis()
         Vector(0.000, 0.000, 1.000)
+
         """
         return cls(0.0, 0.0, 1.0)
 
@@ -389,9 +449,9 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        start : :class:`compas.geometry.Point` or list
+        start : [float, float, float] | :class:`compas.geometry.Point`
             The start point.
-        end : :class:`compas.geometry.Point` or list
+        end : [float, float, float] | :class:`compas.geometry.Point`
             The end point.
 
         Returns
@@ -403,31 +463,11 @@ class Vector(Primitive):
         --------
         >>> Vector.from_start_end([1.0, 0.0, 0.0], [1.0, 1.0, 0.0])
         Vector(0.000, 1.000, 0.000)
+
         """
         v = subtract_vectors(end, start)
         return cls(*v)
 
-    @classmethod
-    def from_data(cls, data):
-        """Construct a vector from a data dict.
-
-        Parameters
-        ----------
-        data : dict
-            The data dictionary.
-
-        Returns
-        -------
-        :class:`compas.geometry.Vector`
-            The vector constructed from the provided data.
-
-        Examples
-        --------
-        >>> Vector.from_data([0.0, 0.0, 1.0])
-        Vector(0.000, 0.000, 1.000)
-        """
-        return cls(*data)
-
     # ==========================================================================
     # static
     # ==========================================================================
@@ -438,9 +478,13 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        collection : list of :class:`compas.geometry.Vector`
+        collection : list[[float, float, float] | :class:`compas.geometry.Vector`]
             The collection of vectors.
 
+        Returns
+        -------
+        None
+
         Examples
         --------
         >>> from compas.geometry import Rotation
@@ -453,6 +497,7 @@ class Vector(Primitive):
         Vector(0.000, 1.000, 0.000)
         >>> u is v
         True
+
         """
         data = transform_vectors(collection, X)
         for vector, xyz in zip(collection, data):
@@ -466,12 +511,12 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        collection : list of :class:`compas.geometry.Vector`
+        collection : list[[float, float, float] | :class:`compas.geometry.Vector`]
             The collection of vectors.
 
         Returns
         -------
-        list of :class:`compas.geometry.Vector`
+        list[:class:`compas.geometry.Vector`]
             The transformed vectors.
 
         Examples
@@ -486,6 +531,7 @@ class Vector(Primitive):
         Vector(0.000, 1.000, 0.000)
         >>> u is v
         False
+
         """
         vectors = [vector.copy() for vector in collection]
         Vector.transform_collection(vectors, X)
@@ -497,18 +543,19 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        vectors : list
+        vectors : list[[float, float, float] | :class:`compas.geometry.Vector`]
             A list of vectors.
 
         Returns
         -------
-        list
+        list[float]
             A list of lengths.
 
         Examples
         --------
         >>> Vector.length_vectors([[1.0, 0.0, 0.0], [2.0, 0.0, 0.0]])
         [1.0, 2.0]
+
         """
         return [length_vector(vector) for vector in vectors]
 
@@ -518,7 +565,7 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        vectors : list
+        vectors : list[[float, float, float] | :class:`compas.geometry.Vector`]
             A list of vectors.
 
         Returns
@@ -530,6 +577,7 @@ class Vector(Primitive):
         --------
         >>> Vector.sum_vectors([[1.0, 0.0, 0.0], [2.0, 0.0, 0.0]])
         Vector(3.000, 0.000, 0.000)
+
         """
         return Vector(* [sum(axis) for axis in zip(* vectors)])
 
@@ -539,20 +587,21 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        left : list
+        left : list[[float, float, float] | :class:`compas.geometry.Vector`]
             A list of vectors.
-        right : list
+        right : list[[float, float, float] | :class:`compas.geometry.Vector`]
             A list of vectors.
 
         Returns
         -------
-        list
+        list[float]
             A list of dot products.
 
         Examples
         --------
         >>> Vector.dot_vectors([[1.0, 0.0, 0.0], [2.0, 0.0, 0.0]], [[1.0, 0.0, 0.0], [2.0, 0.0, 0.0]])
         [1.0, 4.0]
+
         """
         return [Vector.dot(u, v) for u, v in zip(left, right)]
 
@@ -562,20 +611,21 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        left : list
+        left : list[[float, float, float] | :class:`compas.geometry.Vector`]
             A list of vectors.
-        right : list
+        right : list[[float, float, float] | :class:`compas.geometry.Vector`]
             A list of vectors.
 
         Returns
         -------
-        list
+        list[:class:`compas.geometry.Vector`]
             A list of cross products.
 
         Examples
         --------
         >>> Vector.cross_vectors([[1.0, 0.0, 0.0], [2.0, 0.0, 0.0]], [[0.0, 1.0, 0.0], [0.0, 0.0, 2.0]])
         [Vector(0.000, 0.000, 1.000), Vector(0.000, -4.000, 0.000)]
+
         """
         return [Vector.cross(u, v) for u, v in zip(left, right)]
 
@@ -585,20 +635,21 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        left : list
+        left : list[[float, float, float] | :class:`compas.geometry.Vector`]
             A list of vectors.
-        right : list
+        right : list[[float, float, float] | :class:`compas.geometry.Vector`]
             A list of vectors.
 
         Returns
         -------
-        list
+        list[float]
             A list of angle pairs.
 
         Examples
         --------
         >>> Vector.angles_vectors([[1.0, 0.0, 0.0], [2.0, 0.0, 0.0]], [[0.0, 1.0, 0.0], [0.0, 0.0, 2.0]])
         [(1.5707963267948966, 4.71238898038469), (1.5707963267948966, 4.71238898038469)]
+
         """
         return [angles_vectors(u, v) for u, v in zip(left, right)]
 
@@ -608,20 +659,21 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        left : list
+        left : list[[float, float, float] | :class:`compas.geometry.Vector`]
             A list of vectors.
-        right : list
+        right : list[[float, float, float] | :class:`compas.geometry.Vector`]
             A list of vectors.
 
         Returns
         -------
-        list
+        list[float]
             A list of angles.
 
         Examples
         --------
         >>> Vector.angle_vectors([[1.0, 0.0, 0.0], [2.0, 0.0, 0.0]], [[0.0, 1.0, 0.0], [0.0, 0.0, 2.0]])
         [1.5707963267948966, 1.5707963267948966]
+
         """
         return [angle_vectors(u, v) for u, v in zip(left, right)]
 
@@ -645,6 +697,7 @@ class Vector(Primitive):
         True
         >>> u is v
         False
+
         """
         cls = type(self)
         return cls(self.x, self.y, self.z)
@@ -656,12 +709,17 @@ class Vector(Primitive):
     def unitize(self):
         """Scale this vector to unit length.
 
+        Returns
+        -------
+        None
+
         Examples
         --------
         >>> u = Vector(1.0, 2.0, 3.0)
         >>> u.unitize()
         >>> u.length
         1.0
+
         """
         length = self.length
         self.x = self.x / length
@@ -684,6 +742,7 @@ class Vector(Primitive):
         False
         >>> v.length == 1.0
         True
+
         """
         v = self.copy()
         v.unitize()
@@ -692,8 +751,12 @@ class Vector(Primitive):
     def invert(self):
         """Invert the direction of this vector
 
-        Note
-        ____
+        Returns
+        -------
+        None
+
+        Notes
+        -----
         a negation of a vector is similar to inverting a vector
 
         Examples
@@ -708,12 +771,17 @@ class Vector(Primitive):
         True
         >>> v == --v
         True
+
         """
         self.scale(-1.)
 
     def inverted(self):
         """Returns a inverted copy of this vector
 
+        Returns
+        -------
+        :class:`compas.geometry.Vector`
+
         Examples
         --------
         >>> u = Vector(1.0, 0.0, 0.0)
@@ -721,6 +789,7 @@ class Vector(Primitive):
         >>> w = u + v
         >>> w.length
         0.0
+
         """
         return self.scaled(-1.)
 
@@ -732,12 +801,17 @@ class Vector(Primitive):
         n : float
             The scaling factor.
 
+        Returns
+        -------
+        None
+
         Examples
         --------
         >>> u = Vector(1.0, 0.0, 0.0)
         >>> u.scale(3.0)
         >>> u.length
         3.0
+
         """
         self.x *= n
         self.y *= n
@@ -764,6 +838,7 @@ class Vector(Primitive):
         1.0
         >>> v.length
         3.0
+
         """
         v = self.copy()
         v.scale(n)
@@ -774,7 +849,7 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        other : :class:`compas.geometry.Vector` or list
+        other : [float, float, float] | :class:`compas.geometry.Vector`
             The other vector.
 
         Returns
@@ -788,6 +863,7 @@ class Vector(Primitive):
         >>> v = Vector(0.0, 1.0, 0.0)
         >>> u.dot(v)
         0.0
+
         """
         return dot_vectors(self, other)
 
@@ -796,7 +872,7 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        other : :class:`compas.geometry.Vector` or list
+        other : [float, float, float] | :class:`compas.geometry.Vector`
             The other vector.
 
         Returns
@@ -810,6 +886,7 @@ class Vector(Primitive):
         >>> v = Vector(0.0, 1.0, 0.0)
         >>> u.cross(v)
         Vector(0.000, 0.000, 1.000)
+
         """
         return Vector(* cross_vectors(self, other))
 
@@ -818,7 +895,7 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        other : :class:`compas.geometry.Vector` or list
+        other : [float, float, float] | :class:`compas.geometry.Vector`
             The other vector.
 
         Returns
@@ -832,6 +909,7 @@ class Vector(Primitive):
         >>> v = Vector(0.0, 1.0, 0.0)
         >>> u.angle(v) == 0.5 * math.pi
         True
+
         """
         return angle_vectors(self, other)
 
@@ -840,9 +918,9 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        other : :class:`compas.geometry.Vector` or list
+        other : [float, float, float] | :class:`compas.geometry.Vector`
             The other vector.
-        normal : :class:`compas.geometry.Vector` or list
+        normal : [float, float, float] | :class:`compas.geometry.Vector`
             The plane's normal spanned by this and the other vector.
 
         Returns
@@ -858,6 +936,7 @@ class Vector(Primitive):
         True
         >>> u.angle_signed(v, Vector(0.0, 0.0, -1.0)) == -0.5 * math.pi
         True
+
         """
         return angle_vectors_signed(self, other, normal)
 
@@ -866,12 +945,12 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        other : :class:`compas.geometry.Vector` or list
+        other : [float, float, float] | :class:`compas.geometry.Vector`
             The other vector.
 
         Returns
         -------
-        tuple of float
+        tuple[float, float]
             The angles between the two vectors, with the smallest angle first.
 
         Examples
@@ -880,6 +959,7 @@ class Vector(Primitive):
         >>> v = Vector(0.0, 1.0, 0.0)
         >>> u.angles(v)[0] == 0.5 * math.pi
         True
+
         """
         return angles_vectors(self, other)
 
@@ -888,9 +968,13 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        T : :class:`compas.geometry.Transformation` or list of list
+        T : :class:`compas.geometry.Transformation` | list[list[float]]
             The transformation.
 
+        Returns
+        -------
+        None
+
         Examples
         --------
         >>> from compas.geometry import Rotation
@@ -899,6 +983,7 @@ class Vector(Primitive):
         >>> u.transform(R)
         >>> u
         Vector(0.000, 1.000, 0.000)
+
         """
         point = transform_vectors([self], T)[0]
         self.x = point[0]
@@ -910,7 +995,7 @@ class Vector(Primitive):
 
         Parameters
         ----------
-        T : :class:`compas.geometry.Transformation` or list of list
+        T : :class:`compas.geometry.Transformation` | list[list[float]]
             The transformation.
 
         Returns
@@ -926,6 +1011,7 @@ class Vector(Primitive):
         >>> v = u.transformed(R)
         >>> v
         Vector(0.000, 1.000, 0.000)
+
         """
         vector = self.copy()
         vector.transform(T)
